package com.flyfish.framework.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.domain.po.Role;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserStatus;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.utils.CopyUtils;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.*;

/**
 * 客户端用户详情
 *
 * @author wangyu
 */
@Getter
@Setter
public class AdminUserDetails implements UserDetails, IUser, AuthorizedUserDetails {

    private static final long serialVersionUID = -2441854985340378429L;

    private static final List<UserType> adminTypes = Arrays.asList(UserType.ADMIN, UserType.SUPER_ADMIN);
    /**
     * 主键
     */
    protected String id;
    /**
     * 编号
     */
    protected String code;
    /**
     * 名称
     */
    protected String name;
    /**
     * 用户类型
     */
    private UserType type;
    /**
     * 用户状态
     */
    private UserStatus status;
    /**
     * 冗余的电话号码
     */
    private String phone;
    /**
     * 用户名
     */
    private String username;
    /**
     * 密码
     */
    private String password;
    /**
     * 是否启用
     */
    private Boolean enable;
    /**
     * 能否登录移动端
     */
    private Boolean app;
    /**
     * 有效期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date validDate;
    /**
     * 可操作校区
     */
    private List<Department> departments;
    /**
     * 所属角色
     */
    private List<Role> roles;
    /**
     * 微信openId
     */
    private String openId;
    /**
     * 当前用户的鉴权标记
     */
    @Getter(AccessLevel.NONE)
    private String authority;
    /**
     * 查询冗余，标记用户信息
     */
    private Object detail;
    /**
     * 权限code列表
     */
    private Set<String> authorityCodes;
    /**
     * 可见的部门codes
     */
    private Set<String> visibleDeparts;

    /**
     * 判断是否是管理员
     *
     * @param user 用户
     * @return 结果
     */
    public static boolean isAdmin(User user) {
        return adminTypes.contains(user.getType());
    }

    @Override
    @JsonIgnore
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return null;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonExpired() {
        return adminTypes.contains(getType());
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonLocked() {
        return isAccountNonExpired();
    }

    @Override
    @JsonIgnore
    public boolean isCredentialsNonExpired() {
        return null == validDate || validDate.after(new Date());
    }

    @JsonIgnore
    @Override
    public boolean isEnabled() {
        return BooleanUtils.isTrue(getEnable());
    }

    public User toUser() {
        return CopyUtils.copyProps(this, new User());
    }

    @Override
    public String getAuthority() {
        if (StringUtils.isBlank(authority) && null != departments) {
            authority = departments.stream().findFirst().map(Domain::getId).orElse(null);
        }
        return authority;
    }
}
