package com.flyfish.framework.service;

import com.flyfish.framework.domain.UserQo;
import com.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.repository.UserRepository;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.ArrayList;

/**
 * 异步用户service
 *
 * @author wangyu
 */
@Service
public class UserService extends BaseReactiveServiceImpl<User> implements UserFindService {

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    @Override
    public Mono<User> findByUsername(String username) {
        return ((UserRepository) repository).findByUsername(username);
    }

    /**
     * 分页查询，这里使用作用域查询
     *
     * @param query 查询实体
     * @return 结果
     */
    @Override
    public Mono<Page<User>> getPageList(Qo<User> query) {
        // 非超级管理员，永远返回作用域的部门人员
        if (query.getUser().getType() != UserType.SUPER_ADMIN) {
            AuthorizedUserDetails userDetails = (AuthorizedUserDetails) query.getUser();
            UserQo qo = (UserQo) query;
            if (CollectionUtils.isEmpty(qo.getDepartments())) {
                qo.setDepartments(new ArrayList<>(userDetails.getVisibleDeparts()));
            }
        }
        return super.getPageList(query);
    }
}
