package com.flyfish.framework.configuration.redis;

import com.flyfish.framework.utils.ReactiveRedisOperations;
import org.springframework.context.annotation.Bean;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.ReactiveRedisTemplate;
import org.springframework.data.redis.core.ReactiveStringRedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializationContext;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

public class ReactiveRedisConfig {

    @Bean
    public ReactiveStringRedisTemplate reactiveStringRedisTemplate(LettuceConnectionFactory lettuceConnectionFactory) {
        return new ReactiveStringRedisTemplate(lettuceConnectionFactory);
    }

    /**
     * RedisTemplate配置
     *
     * @param lettuceConnectionFactory 支持异步的连接池
     * @return 结果
     */
    @Bean
    public ReactiveRedisTemplate<String, Object> reactiveRedisTemplate(LettuceConnectionFactory lettuceConnectionFactory) {
        Jackson2JsonRedisSerializer<Object> jacksonSerializer = JacksonRedisSerializerFactory.produce();
        RedisSerializer<String> stringSerializer = new StringRedisSerializer();
        RedisSerializationContext<String, Object> serializationContext = RedisSerializationContext
                .<String, Object>newSerializationContext()
                // key序列化
                .key(stringSerializer)
                // value序列化
                .value(jacksonSerializer)
                // Hash key序列化
                .hashKey(stringSerializer)
                // Hash value序列化
                .hashValue(jacksonSerializer)
                .build();
        // 配置redisTemplate
        return new ReactiveRedisTemplate<>(lettuceConnectionFactory, serializationContext);
    }

    @Bean
    public ReactiveRedisOperations reactiveRedisOperations(ReactiveStringRedisTemplate reactiveStringRedisTemplate,
                                                           ReactiveRedisTemplate<String, Object> reactiveRedisTemplate) {
        return new ReactiveRedisOperations(reactiveRedisTemplate, reactiveStringRedisTemplate);
    }

}
